==> gitlab_16.11.6.orig.tar.bz2: gitlab-foss-v16.11.6/lib/gitlab/database_importers/work_items/base_type_importer.rb <==
# frozen_string_literal: true

module Gitlab
  module DatabaseImporters
    module WorkItems
      module BaseTypeImporter
        WIDGET_NAMES = {
          assignees: 'Assignees',
          labels: 'Labels',
          description: 'Description',
          hierarchy: 'Hierarchy',
          start_and_due_date: 'Start and due date',
          milestone: 'Milestone',
          notes: 'Notes',
          iteration: 'Iteration',
          weight: 'Weight',
          health_status: 'Health status',
          progress: 'Progress',
          status: 'Status',
          requirement_legacy: 'Requirement legacy',
          test_reports: 'Test reports',
          notifications: 'Notifications',
          current_user_todos: 'Current user todos',
          award_emoji: 'Award emoji',
          linked_items: 'Linked items',
          color: 'Color',
          rolledup_dates: 'Rolledup dates',
          participants: 'Participants',
          time_tracking: 'Time tracking',
          designs: 'Designs',
          development: 'Development'
        }.freeze

        WIDGETS_FOR_TYPE = {
          issue: [
            :assignees,
            :labels,
            :description,
            :hierarchy,
            :start_and_due_date,
            :milestone,
            :notes,
            :iteration,
            :weight,
            :health_status,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants,
            :time_tracking,
            :designs,
            :development
          ],
          incident: [
            :assignees,
            :description,
            :hierarchy,
            :notes,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants,
            :time_tracking,
            :development
          ],
          test_case: [
            :description,
            :notes,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants,
            :time_tracking
          ],
          requirement: [
            :description,
            :notes,
            :status,
            :requirement_legacy,
            :test_reports,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants,
            :time_tracking
          ],
          task: [
            :assignees,
            :labels,
            :description,
            :hierarchy,
            :start_and_due_date,
            :milestone,
            :notes,
            :iteration,
            :weight,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants,
            :time_tracking
          ],
          objective: [
            :assignees,
            :labels,
            :description,
            :hierarchy,
            :milestone,
            :notes,
            :health_status,
            :progress,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants
          ],
          key_result: [
            :assignees,
            :labels,
            :description,
            :hierarchy,
            :start_and_due_date,
            :notes,
            :health_status,
            :progress,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants
          ],
          epic: [
            :assignees,
            :description,
            :hierarchy,
            :labels,
            :notes,
            :start_and_due_date,
            :health_status,
            :status,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :color,
            :rolledup_dates,
            :participants,
            :time_tracking
          ],
          ticket: [
            :assignees,
            :labels,
            :description,
            :hierarchy,
            :start_and_due_date,
            :milestone,
            :notes,
            :iteration,
            :weight,
            :health_status,
            :notifications,
            :current_user_todos,
            :award_emoji,
            :linked_items,
            :participants,
            :time_tracking
          ]
        }.freeze

        def self.upsert_types
          current_time = Time.current

          base_types = ::WorkItems::Type::BASE_TYPES.map do |type, attributes|
            attributes.slice(:name, :icon_name)
                      .merge(created_at: current_time, updated_at: current_time, base_type: type)
          end

          ::WorkItems::Type.upsert_all(
            base_types,
            unique_by: :idx_work_item_types_on_namespace_id_and_name_null_namespace
          )

          upsert_widgets
        end

        def self.upsert_widgets
          type_ids_by_name = ::WorkItems::Type.default.pluck(:name, :id).to_h # rubocop: disable CodeReuse/ActiveRecord

          widgets = WIDGETS_FOR_TYPE.flat_map do |type_sym, widget_syms|
            type_name = ::WorkItems::Type::TYPE_NAMES[type_sym]

            widget_syms.map do |widget_sym|
              {
                work_item_type_id: type_ids_by_name[type_name],
                name: WIDGET_NAMES[widget_sym],
                widget_type: ::WorkItems::WidgetDefinition.widget_types[widget_sym]
              }
            end
          end

          ::WorkItems::WidgetDefinition.upsert_all(
            widgets,
            unique_by: :index_work_item_widget_definitions_on_default_witype_and_name
          )
        end
      end
    end
  end
end

==> gitlab_17.3.5.orig.tar.bz2: gitlab-foss-v17.3.5/lib/gitlab/database_importers/work_items/base_type_importer.rb <==
# frozen_string_literal: true

module Gitlab
  module DatabaseImporters
    module WorkItems
      module BaseTypeImporter
        WIDGET_NAMES = {
          assignees: 'Assignees',
          labels: 'Labels',
          description: 'Description',
          hierarchy: 'Hierarchy',
          start_and_due_date: 'Start and due date',
          milestone: 'Milestone',
          notes: 'Notes',
          iteration: 'Iteration',
          weight: 'Weight',
          health_status: 'Health status',
          progress: 'Progress',
          status: 'Status',
          requirement_legacy: 'Requirement legacy',
          test_reports: 'Test reports',
          notifications: 'Notifications',
          current_user_todos: 'Current user todos',
          award_emoji: 'Award emoji',
          linked_items: 'Linked items',
          color: 'Color',
          rolledup_dates: 'Rolledup dates',
          participants: 'Participants',
          time_tracking: 'Time tracking',
          designs: 'Designs',
          development: 'Development',
          crm_contacts: 'CRM contacts'
        }.freeze

        WIDGETS_FOR_TYPE = {
          issue: [
            :assignees,
            :award_emoji,
            :crm_contacts,
            :current_user_todos,
            :description,
            :designs,
            :development,
            :health_status,
            :hierarchy,
            :iteration,
            :labels,
            :linked_items,
            :milestone,
            :notes,
            :notifications,
            :participants,
            :start_and_due_date,
            :time_tracking,
            [:weight, { editable: true, rollup: false }]
          ],
          incident: [
            :assignees,
            :award_emoji,
            :crm_contacts,
            :current_user_todos,
            :description,
            :development,
            :hierarchy,
            :linked_items,
            :notes,
            :notifications,
            :participants,
            :time_tracking
          ],
          test_case: [
            :award_emoji,
            :current_user_todos,
            :description,
            :linked_items,
            :notes,
            :notifications,
            :participants,
            :time_tracking
          ],
          requirement: [
            :award_emoji,
            :current_user_todos,
            :description,
            :linked_items,
            :notes,
            :notifications,
            :participants,
            :requirement_legacy,
            :status,
            :test_reports,
            :time_tracking
          ],
          task: [
            :assignees,
            :award_emoji,
            :crm_contacts,
            :current_user_todos,
            :description,
            :development,
            :hierarchy,
            :iteration,
            :labels,
            :linked_items,
            :milestone,
            :notes,
            :notifications,
            :participants,
            :start_and_due_date,
            :time_tracking,
            [:weight, { editable: true, rollup: false }]
          ],
          objective: [
            :assignees,
            :award_emoji,
            :current_user_todos,
            :description,
            :health_status,
            :hierarchy,
            :labels,
            :linked_items,
            :milestone,
            :notes,
            :notifications,
            :participants,
            :progress
          ],
          key_result: [
            :assignees,
            :award_emoji,
            :current_user_todos,
            :description,
            :health_status,
            :hierarchy,
            :labels,
            :linked_items,
            :notes,
            :notifications,
            :participants,
            :start_and_due_date,
            :progress
          ],
          epic: [
            :assignees,
            :award_emoji,
            :color,
            :crm_contacts,
            :current_user_todos,
            :description,
            :health_status,
            :hierarchy,
            :labels,
            :linked_items,
            :notes,
            :notifications,
            :participants,
            :rolledup_dates,
            :start_and_due_date,
            :status,
            :time_tracking,
            [:weight, { editable: false, rollup: true }]
          ],
          ticket: [
            :assignees,
            :award_emoji,
            :crm_contacts,
            :current_user_todos,
            :description,
            :health_status,
            :hierarchy,
            :iteration,
            :labels,
            :linked_items,
            :milestone,
            :notes,
            :notifications,
            :participants,
            :start_and_due_date,
            :time_tracking,
            [:weight, { editable: true, rollup: false }]
          ]
        }.freeze

        def self.upsert_types
          current_time = Time.current

          base_types = ::WorkItems::Type::BASE_TYPES.map do |type, attributes|
            attributes.slice(:name, :icon_name)
                      .merge(created_at: current_time, updated_at: current_time, base_type: type)
          end

          ::WorkItems::Type.upsert_all(
            base_types,
            unique_by: :idx_work_item_types_on_namespace_id_and_name_null_namespace
          )

          upsert_widgets
        end

        def self.upsert_widgets
          type_ids_by_name = ::WorkItems::Type.pluck(:name, :id).to_h # rubocop: disable CodeReuse/ActiveRecord

          widgets = WIDGETS_FOR_TYPE.flat_map do |type_sym, widget_syms|
            type_name = ::WorkItems::Type::TYPE_NAMES[type_sym]

            widget_syms.map do |widget_sym|
              widget_sym, widget_options = widget_sym if widget_sym.is_a?(Array)

              {
                work_item_type_id: type_ids_by_name[type_name],
                name: WIDGET_NAMES[widget_sym],
                widget_type: ::WorkItems::WidgetDefinition.widget_types[widget_sym],
                widget_options: widget_options
              }
            end
          end

          ::WorkItems::WidgetDefinition.upsert_all(
            widgets,
            unique_by: :index_work_item_widget_definitions_on_default_witype_and_name
          )
        end
      end
    end
  end
end
